/*
 * Copyright (c) 2008, 2009  Nokia Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#include "exa.h"
#include "sgx_exa.h"
#include "sgx_exa_user.h"

void sgx_exa_copy_region(DrawablePtr draw, RegionPtr reg, DrawablePtr src_draw,
			DrawablePtr dst_draw)
{
        ScreenPtr screen = dst_draw->pScreen;
        RegionPtr copy_clip = REGION_CREATE(screen, NULL, 0);
        GCPtr gc;

        gc = GetScratchGC(dst_draw->depth, screen);
        REGION_COPY(screen, copy_clip, reg);
        (*gc->funcs->ChangeClip)(gc, CT_REGION, copy_clip, 0);
        ValidateGC(dst_draw, gc);

        (*gc->ops->CopyArea)(src_draw, dst_draw, gc, 0, 0, draw->width,
                                draw->height, 0, 0);

        FreeScratchGC(gc);
}

void sgx_exa_set_reg(const DrawablePtr draw, RegionRec *reg)
{
	BoxRec box;

	box.x1 = 0;
	box.y1 = 0;
	box.x2 = draw->width;
	box.y2 = draw->height;

	REGION_INIT(draw->pScreen, reg, &box, 0);
}

static PixmapPtr sgx_exa_create_pixmap(DrawablePtr draw, unsigned fmt,
					PVR2DMEMINFO *mem_info)
{
	ScreenPtr screen = draw->pScreen;
	PixmapPtr res;

	res = (*screen->CreatePixmap)(screen, draw->width, draw->height,
					(fmt != 0) ? fmt : draw->depth,
					SGX_EXA_CREATE_PIXMAP_FLIP);

	if (res) {
		struct PVR2DPixmap *priv = exaGetPixmapDriverPrivate(res);

		priv->pvr2dmem = mem_info;
		priv->shmaddr = mem_info->pBase;
	}

	return res;
}

int sgx_exa_copy_from_buf(DrawablePtr draw, RegionPtr reg, unsigned fmt,
			  PVR2DMEMINFO *src, DrawablePtr dst)
{
	PixmapPtr src_pix = sgx_exa_create_pixmap(draw, fmt, src);

	if (!src_pix)
		return 1;

	sgx_exa_copy_region(draw, reg, &src_pix->drawable, dst);

	(*draw->pScreen->DestroyPixmap)(src_pix);

	return 0;
}

int sgx_exa_copy_to_buf(DrawablePtr draw, RegionPtr reg, unsigned fmt,
			DrawablePtr src, PVR2DMEMINFO *dst)
{
	PixmapPtr dst_pix = sgx_exa_create_pixmap(draw, fmt, dst);

	if (!dst_pix)
		return 1;

	sgx_exa_copy_region(draw, reg, src, &dst_pix->drawable);

	(*draw->pScreen->DestroyPixmap)(dst_pix);

	return 0;
}

int sgx_exa_copy_buf_to_buf(DrawablePtr draw, RegionPtr reg,
			    unsigned src_fmt, unsigned dst_fmt,
			    PVR2DMEMINFO *src, PVR2DMEMINFO *dst)
{
	int result = 0;
	PixmapPtr src_pix = sgx_exa_create_pixmap(draw, src_fmt, src);
	PixmapPtr dst_pix = sgx_exa_create_pixmap(draw, dst_fmt, dst);

	if (src_pix && dst_pix) {
		sgx_exa_copy_region(draw, reg, &src_pix->drawable,
				    &dst_pix->drawable);
	} else {
		result = 1;
	}

	if (src_pix)
		(*draw->pScreen->DestroyPixmap)(src_pix);

	if (dst_pix)
		(*draw->pScreen->DestroyPixmap)(dst_pix);

	return result;
}

/* EOF */
